mod api;
mod error;
mod metrics;
mod server;

pub use error::Result;

use std::{net::SocketAddr, path::PathBuf};

use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt};

/// Name of the directory where static sites are stored inside the data directory
const STATIC_DIR_NAME: &str = "static";

const REDIRECTS: [(&str, &str); 6] = [
    ("/github", "https://github.com/ChewingBever"),
    ("/gitea", "https://git.rustybever.be/Chewing_Bever"),
    ("/gitlab", "https://gitlab.com/Chewing_Bever"),
    ("/codeberg", "https://codeberg.org/Chewing_Bever"),
    ("/matrix", "https://matrix.to/#/@jef:rustybever.be"),
    ("/aur", "https://aur.archlinux.org/account/Chewing_Bever"),
];

#[tokio::main]
async fn main() {
    // Enable tracing
    tracing_subscriber::registry()
        .with(tracing_subscriber::EnvFilter::new(
            std::env::var("RUST_LOG").unwrap_or_else(|_| "site=debug,tower_http=debug".into()),
        ))
        .with(tracing_subscriber::fmt::layer())
        .init();

    // Get required variables from env vars
    let api_key = std::env::var("API_KEY").expect("No API_KEY was provided.");
    let data_dir = std::env::var("DATA_DIR").expect("No DATA_DIR was provided.");
    let static_dir = format!("{}/{}", data_dir, STATIC_DIR_NAME);

    std::fs::create_dir_all(&static_dir);

    // Initialize metrics
    let recorder_handle = metrics::setup_metrics_recorder();
    let app = server::app(PathBuf::from(static_dir), &REDIRECTS);

    // Each static site gets mounted explicitely so that the default site can be used as fallback
    // Each entry is of the form (route, static dir name)
    let sites = [
        ("/docs/vieter", "docs-vieter"),
        ("/api-docs/vieter", "api-docs-vieter"),
        ("/man/vieter", "man-vieter"),
    ];

    let addr = SocketAddr::from(([0, 0, 0, 0], 3000));
    tracing::debug!("listening on {}", addr);
    axum::Server::bind(&addr)
        .serve(app.into_make_service())
        .await
        .unwrap();
}
